@@ -10,7 +10,7 @@
 #define _R_FUNCTIONS_H_
 
 #include <Rcpp.h>
-#include "mlnetwork.h"
+#include "multinet.h"
 #include <unordered_set>
 #include <vector>
 
@@ -39,7 +39,7 @@ class REvolutionModel {
 
 RMLNetwork emptyMultilayer(const std::string& name);
 void renameMultilayer(RMLNetwork& rmnet, const std::string& new_name);
-RMLNetwork readMultilayer(const std::string& input_file, const std::string& name, char sep);
+RMLNetwork readMultilayer(const std::string& input_file, const std::string& name, char sep, bool node_aligned);
 void writeMultilayer(const RMLNetwork& mnet, const std::string& output_file, const std::string& format, const CharacterVector& layer_names, char sep, bool merge_actors, bool all_actors);
 
 REvolutionModel ba_evolution_model(int m0, int m);
@@ -50,14 +50,16 @@ RMLNetwork growMultiplex(int num_actors, long num_of_steps, const GenericVector&
 
 CharacterVector layers(const RMLNetwork& mnet);
 CharacterVector actors(const RMLNetwork& mnet, const CharacterVector& layer_names);
-CharacterMatrix nodes(const RMLNetwork& mnet, const CharacterVector& layer_names);
-CharacterMatrix edges(const RMLNetwork& mnet, const CharacterVector& layer_names1, const CharacterVector& layer_names2);
+DataFrame nodes(const RMLNetwork& mnet, const CharacterVector& layer_names);
+DataFrame edges(const RMLNetwork& mnet, const CharacterVector& layer_names1, const CharacterVector& layer_names2);
+
+DataFrame edges_idx(const RMLNetwork& rmnet);
 
 int numLayers(const RMLNetwork& mnet);
 long numActors(const RMLNetwork& mnet, const CharacterVector& layers);
 long numNodes(const RMLNetwork& mnet, const CharacterVector& layers);
 long numEdges(const RMLNetwork& mnet, const CharacterVector& layer_names1, const CharacterVector& layer_names2);
-CharacterMatrix isDirected(const RMLNetwork& mnet, const CharacterVector& layer_names1, const CharacterVector& layer_names2);
+DataFrame isDirected(const RMLNetwork& mnet, const CharacterVector& layer_names1, const CharacterVector& layer_names2);
 
 std::unordered_set<std::string> actor_neighbors(const RMLNetwork& rmnet, const std::string& actor_name, const CharacterVector& layer_names, const std::string& mode_name);
 std::unordered_set<std::string> actor_xneighbors(const RMLNetwork& rmnet, const std::string& actor_name, const CharacterVector& layer_names, const std::string& mode_name);
@@ -66,20 +68,20 @@ std::unordered_set<std::string> actor_xneighbors(const RMLNetwork& rmnet, const
 
 void addLayers(RMLNetwork& rmnet, const CharacterVector& layer_names, const LogicalVector& directed);
 void addActors(RMLNetwork& rmnet, const CharacterVector& actor_names);
-void addNodes(RMLNetwork& rmnet, const CharacterVector& nodes);
-void addEdges(RMLNetwork& rmnet, const CharacterVector& edges);
-void setDirected(const RMLNetwork& rmnet, const CharacterVector& layers, bool directed);
+void addNodes(RMLNetwork& rmnet, const DataFrame& nodes);
+void addEdges(RMLNetwork& rmnet, const DataFrame& edges);
+void setDirected(const RMLNetwork& rmnet, const DataFrame& directionalities);
 
 void deleteLayers(RMLNetwork& rmnet, const CharacterVector& layer_names);
 void deleteActors(RMLNetwork& rmnet, const CharacterVector& actor_names);
-void deleteNodes(RMLNetwork& rmnet, const CharacterVector& nodes);
-void deleteEdges(RMLNetwork& rmnet, const CharacterVector& edges);
+void deleteNodes(RMLNetwork& rmnet, const DataFrame& nodes);
+void deleteEdges(RMLNetwork& rmnet, const DataFrame& edges);
 
 
 void newAttributes(RMLNetwork& rmnet, const CharacterVector& attribute_names, const std::string& type, const std::string& target, const std::string& layer_name, const std::string& layer_name1, const std::string& layer_name2);
-GenericMatrix getAttributes(const RMLNetwork& rmnet, const std::string& target);
-GenericMatrix getValues(RMLNetwork& rmnet, const std::string& attribute_name, const CharacterVector& actor_names, const CharacterVector& layer_names, const CharacterVector& node_matrix, const CharacterVector& edge_matrix);
-void setValues(RMLNetwork& rmnet, const std::string& attribute_name, const CharacterVector& actor_names, const CharacterVector& layer_names, const CharacterVector& node_matrix, const CharacterVector& edge_matrix, const GenericVector& values);
+DataFrame getAttributes(const RMLNetwork& rmnet, const std::string& target);
+DataFrame getValues(RMLNetwork& rmnet, const std::string& attribute_name, const CharacterVector& actor_names, const CharacterVector& layer_names, const DataFrame& node_matrix, const DataFrame& edge_matrix);
+void setValues(RMLNetwork& rmnet, const std::string& attribute_name, const CharacterVector& actor_names, const CharacterVector& layer_names, const DataFrame& node_matrix, const DataFrame& edge_matrix, const GenericVector& values);
 
 // TRANSFORMATION
 
@@ -96,35 +98,25 @@ NumericVector xneighborhood_ml(const RMLNetwork& mnet, const CharacterVector& ac
 NumericVector connective_redundancy_ml(const RMLNetwork& mnet, const CharacterVector& actor_names, const CharacterVector& layer_names, const std::string& type);
 NumericVector relevance_ml(const RMLNetwork& mnet, const CharacterVector& actor_names, const CharacterVector& layer_names, const std::string& type);
 NumericVector xrelevance_ml(const RMLNetwork& mnet, const CharacterVector& actor_names, const CharacterVector& layer_names, const std::string& type);
-double similarity_ml(const RMLNetwork& rmnet, const std::string& layer1, const std::string& layer2, const std::string& method);
-double correlation_ml(const RMLNetwork& rmnet, const std::string& layer1, const std::string& layer2, const std::string& method, const std::string& type);
-GenericMatrix distance_ml(const RMLNetwork& mnet, const std::string& from, const CharacterVector& to, const std::string& method);
+double summary_ml(const RMLNetwork& rmnet, const std::string& layer, const std::string& method, const std::string& type);
+DataFrame comparison_ml(const RMLNetwork& rmnet, const CharacterVector& layer_names, const std::string& method, const std::string& type, int K);
+DataFrame distance_ml(const RMLNetwork& mnet, const std::string& from, const CharacterVector& to, const std::string& method);
 
 // CLUSTERING
 
-GenericMatrix cliquepercolation_ml(const RMLNetwork& mnet, int k, int m1, int m2, int m3);
+DataFrame cliquepercolation_ml(const RMLNetwork& rmnet, int k, int m);
+DataFrame glouvain_ml(const RMLNetwork& rmnet, double gamma, double omega, int limit);
+DataFrame abacus_ml(const RMLNetwork& rmnet, int min_actors, int min_layers);
+DataFrame lart_ml(const RMLNetwork& rmnet, int t, double eps, double gamma);
+
+List to_list(const DataFrame& cs, const RMLNetwork& mnet);
 
 // Layout
 
-GenericMatrix multiforce_ml(const RMLNetwork& mnet, double width, double length, const CharacterVector& layers, const NumericVector& w_in, const NumericVector& w_out, int iterations);
+DataFrame multiforce_ml(const RMLNetwork& mnet, const NumericVector& w_in, const NumericVector& w_out, const NumericVector& gravity, int iterations);
+DataFrame circular_ml(const RMLNetwork& mnet);
 
-
-// Preprocessing
-/*
-void flatten_w(RMLNetworkSharedPtr& mnet, const std::string& new_layer_name, const CharacterVector& layers, bool force_directed, bool force_actors);
-void flatten_or(RMLNetworkSharedPtr& mnet, const std::string& new_layer_name, const CharacterVector& layers, bool force_directed, bool force_actors);
-void project_or(RMLNetworkSharedPtr& mnet, const std::string& new_layer_name, const std::string& layer1, const std::string& layer2);
-*/
 // SPREADING
 // NumericMatrix sir_ml(const RMLNetwork& mnet, double beta, int tau, long num_iterations);
 
-/*
-
-// MEASURE DISTRIBUTIONS
-
-NumericVector degree_dist(const MLNetwork& mnet, const CharacterVector& layers, const std::string& type);
-
-NumericVector neighborhood_dist(const MLNetwork& mnet, const CharacterVector& layers, const std::string& type);
-NumericVector xneighborhood_dist(const MLNetwork& mnet, const CharacterVector& layers, const std::string& type);
-*/
 #endif /* _R_FUNCTIONS_H_ */
